#!/usr/bin/perl
#-------------------------------------------------------------------------------
# Author: Nick Deubert < ndeubert at gmail dot com >
# Desc  : This script gives you a list of which pacakges are installed in which
#         repositories.
#-------------------------------------------------------------------------------
use Getopt::Long;

GetOptions( "debug"     => \$debug,
            "verbose"   => \$verbose,
            "help"      => \$help,
            "conf=s"    => \$prt_conf,
            "obsolete"  => \$obsolete );

if ($help) {
   print "Usage: which_repos [OPTIONS] [pacakge1] [package2]...\n\n";
   print "Options:\n";
   print "\t--obsolete : Only prints the pacakges that are not in any\n";
   print "\t             repositories.\n";
   print "\t--conffile : Used to sepecify a prt-get.conf file other than the\n";
   print "\t             default: /etc/prt-get.conf\n";
   print "\t--verbose  : Prints additional information about the repositories\n";
   print "\t--help     : Prints this\n";
   print "\t--debug    : Spews everything( not useful unless you want to debug\n";
   print "\t             something)\n";
   exit;
}

if ( !$prt_conf ) {
   $prt_conf = "/etc/prt-get.conf";
}
$ports_file = "/tmp/ports.txt";
$inst_file = "/tmp/installed.txt";

system( "ports -l > $ports_file");
open (PORTS_FILE, $ports_file);
@ports_list = <PORTS_FILE>;
close (PORTS_FILE);
unlink ($ports_file);


if ( $#ARGV < 0 ) {
   print "No packages specified, performing check on all...\n" if $verbose;
   system( "prt-get listinst > $inst_file");
   open (INST_FILE, $inst_file);
   @pkg_list = <INST_FILE>;
   close (INST_FILE);
   unlink ($inst_file);
} else {
   @pkg_list = @ARGV;
}
chomp( @pkg_list );

open (PRT_CONF, $prt_conf);
while( <PRT_CONF> ) {
   if ( $_ =~ m/^prtdir\s+\S+\/(\S+)$/ ) {
      push( @prtdirs, $1 );
   }
}
close (PRT_CONF);

foreach $port (@ports_list) {
   my( $repo, $port_name ) = split( '/', $port );
   chomp( $repo );
   chomp( $port_name );
   push( @{$port_table{ $repo }}, $port_name );
}

if ( $verbose or $debug ) {
   foreach $repo (sort keys %port_table) {
      print "Repo: $repo \t size: ", $#{$port_table{$repo}}+1, "\n" if $verbose;

      if ( $debug ) {
         foreach $port (@{$port_table{$repo}}) {
            print $port, "\n";
         }
      }
   }
}

print "Repo priority: @prtdirs\n" if $verbose;

# loop through all pacakges we want to check
foreach $pkg (@pkg_list) {
   my( @potential_repos );

   # loop through ports in the order of their priority
   foreach $prtdir (@prtdirs) {

      # loop through the list of ports in a repo in the hash table
      foreach $port (@{$port_table{$prtdir}}) {
         print "comparing $pkg and $port\n" if $debug;
         $pause = <STDIN> if $debug;
         if ( $pkg eq $port ) {
            print "$pkg could be installed from $prtdir\n" if $debug;
            push( @potential_repos, $prtdir );
            last;
         }
      }
   }
   
   if ( $#potential_repos < 0 ) {
      print "$pkg was not found in any repository.\n";
   } elsif ( !$obsolete ) {
      print "$pkg: @potential_repos\n";
   }
}
